<?php

namespace App\Http\Controllers\CMS;

use App\Enums\PostImageSize;
use App\Http\Controllers\Controller;
use App\Models\Image;
use App\Libraries\ErrorMessage;
use App\Libraries\Status;
use App\Models\Post;
use App\Repositories\ImageRepository;
use App\Repositories\PostCategoryRepository;
use App\Repositories\PostImageRelationshipRepository;
use App\Repositories\PostRepository;
use App\Services\CMS\PostCategoryService;
use App\Services\CMS\PostService;
use App\Services\Common\ImageService;
use App\ViewModels\CMS\Banner\CreateAndEditViwModel;
use App\ViewModels\CMS\Banner\Image\EditViewModel;
use App\ViewModels\CMS\Banner\IndexViewModel;
use App\ViewModels\CMS\TemplateViewModel;
use Exception;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Illuminate\View\View;
use Livewire\Attributes\Validate;
use PhpParser\Node\Stmt\DeclareDeclare;

/**
 * 橫幅，採用 Post 服務
 */
class BannerController extends Controller
{
    /**
     * 顯示橫幅列表
     *
     * @param  Request  $request
     *
     * @return Application|Factory|View
     */
    public function index(Request $request)
    {
        Gate::authorize("bannerViewAny", Post::class);

        $categoryIds = PostCategoryService::getRootCategoriesByParentId(400,
            $request->categoryId, ['id', 'title']);

        $posts = PostService::getPosts2($categoryIds);

        $postCategories = PostCategoryRepository::getCategoriesByParentId(
            400,
            [
                'id',
                'title'
            ]
        );

        $indexViewModel = new  IndexViewModel(
            $request,
            $postCategories,
            $posts,
            $request->tab
        );

        return view("cms.banner.index", $indexViewModel);
    }

    /**
     * 建立橫幅
     *
     * @param  Request  $request
     *
     * @return Application|Factory|View
     */
    public function create(Request $request)
    {
        Gate::authorize("bannerCreate", Post::class);

        $postCategories = PostCategoryRepository::getCategoriesByParentId(
            400,
            [
                'id',
                'title'
            ]
        );

        $createAndEditViwModel = new  CreateAndEditViwModel(
            $request,
            "create",
            $postCategories,
            null,
            null
        );

        return view("cms.banner.create", $createAndEditViwModel);
    }

    /**
     * 新增橫幅
     *
     * @param  Request  $request
     *
     * @return Application|Factory|View
     */
    public function store(Request $request)
    {
        Gate::authorize("bannerCreate", Post::class);

        // 若有上傳封面，取得編號
        if ($request->hasFile('cover')) {
            $result = ImageService::insert(
                Auth::guard("cms")->user()->id,
                $request->file('cover'),
                1, false
            );

            $coverId = $result['data']['image']['id'];
        } else {
            $coverId = null;
        }

        $result = PostService::insert(
            $request->status,
            $coverId,
            $request->title,
            $request['content'],
            $request->summary,
            $request->publishedAt,
            null,
            Auth::guard("cms")->id(), $request->categories
        );

        return $result;
    }

    /**
     * 編輯橫幅
     *
     * @param  Request  $request
     * @param  string  $id
     *
     * @return Application|Factory|View
     */
    public function edit(Request $request, string $id)
    {
        Gate::authorize("bannerUpdate", Post::class);

        $result = PostService::getPost($id, "imageRelationships");
        if (Status::isFails($result)) {
            abort(404);
        }

        $postCategories = PostCategoryRepository::getCategoriesByParentId(
            400,
            [
                'id',
                'title'
            ]
        );

        $editViewModel = new CreateAndEditViwModel(
            $request,
            "edit",
            $postCategories,
            $result['data']['post'],
            null
        );

        return view("cms.banner.create", $editViewModel);
    }

    /**
     * 圖片編輯頁面
     *
     * @param  Request  $request
     * @param  string  $id
     *
     * @return Application|Factory|View
     */
    public function imageEdit(Request $request, $id)
    {
        Gate::authorize("bannerUpdate", Post::class);

        $result = ImageService::getImage($id);
        if (Status::isFails($result)) {
            abort(403, "這張圖片已經失效");
        }

        $editViewModel = new EditViewModel($request, $id,
            $result['data']['image']);
        return view("cms.banner.imageEdit", $editViewModel);
    }

    /**
     * 刪除橫幅
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     * @throws Exception
     */
    public function destroy(Request $request)
    {
        Gate::authorize("bannerDelete", Post::class);

        $result = PostService::delete($request->id);
        if ($result === false) {
            return redirect()->back()
                ->with(Status::bladeFails("您沒有刪除任何一筆資料"));
        }
        return redirect()->back()->with(Status::bladeSuccess("刪除成功"));
    }

    /**
     * 刪除橫幅的畫廊圖片
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageDestroy(Request $request)
    {
        Gate::authorize("bannerDelete", Post::class);

        Validator::make(
            $request->all(),
            [
                'id' => 'required|string',
                'postImageRelationshipId' => 'required|string',
            ]
        )->validate();

        $result = ImageService::deleteById($request->id, $request->postImageRelationshipId);

        if (Status::isFails($result)) {
            return redirect()->back()
                ->with(Status::bladeFails("刪除失敗：{$result['message']}"));
        }

        return redirect()->back()->with(Status::bladeSuccess("刪除成功"));
    }

    /**
     * 使用網址夾帶訊息，導向指定的頁面
     *
     * @param  Request  $request
     * @param  string  $message
     *
     * @return RedirectResponse
     */
    public function redirectMessageIndex(Request $request, $message)
    {
        if (isset($request->redirect)) {
            return redirect($request->redirect)->with(
                Status::bladeSuccess($message)
            );
        }

        return redirect()->route("cms.banner.index")->with(
            Status::bladeSuccess($message)
        );
    }

    /**
     * 更新橫幅
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function update(Request $request)
    {
        Gate::authorize("bannerUpdate", Post::class);

        $result = PostService::update(
            $request,
            $request->id,
            $request->status,
            $request->coverId,
            $request->title,
            $request['content'],
            $request->summary,
            $request->publishedAt,
            null,
            Auth::guard("cms")->id(),
            $request->postCategoryIds
        );

        return $result;
    }

    /**
     * 更新橫幅的畫廊排序
     *
     * @param  Request  $request
     *
     * @return array
     * @throws Exception
     */
    public function gallerySortUpdate(Request $request)
    {
        Gate::authorize("bannerUpdate", Post::class);

        Validator::make(
            $request->all(),
            [
                'postImageRelationshipSort' => 'required|array',
            ]
        )->validate();


        return PostService::updatePostImageRelationshipSortOrder($request->postImageRelationshipSort);
    }

    /**
     * 更新橫幅的畫廊圖片
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageUpdate(Request $request)
    {
        Gate::authorize("bannerUpdate", Post::class);

        $result = PostService::updateImage(
            $request->id,
            $request->title,
            $request->title2,
            $request->description,
            $request->{'content'},
            $request->link,
            $request->callToActionButtonText,
            $request->callToAction
        );

        return redirect()->away(urldecode($request->redirect))->with(
            Status::bladeSuccess("更新成功")
        );
    }

    /**
     * 上傳橫幅的畫廊圖片
     *
     * @param  Request  $request
     *
     * @return array
     */
    public function imageStore(Request $request)
    {
        Gate::authorize("bannerCreate", Post::class);

        Validator::make(
            $request->all(),
            [
                'id' => 'required|string',
                'images' => 'required|array',
                'imageId' => 'nullable|string',
                'type' => 'required|in:desktop,tablet,mobile',
            ]
        )->validate();

        $images = $request->file('images');


        // 如果沒有 imageId，代表是新增
        if (is_null($request->imageId)) {
            $result = PostService::insertImages(
                $request->id,
                $images,
                PostImageSize::from($request->type),
            );
        } // 如果有 imageId，代表是添加其他尺寸，尺寸的判斷可以由 type 參數判斷
        else {
            $result = PostService::updateElseSizeImage(
                $request->id,
                $request->imageId,
                $images[0],
                PostImageSize::from($request->type),
            );
        }

        return $result;
    }

}
