<?php

namespace App\Http\Controllers\CMS;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\Coupon;
use App\Models\User;
use App\Repositories\CouponUserRelationshipRepository;
use App\Repositories\UserRepository;
use App\ViewModels\CMS\Coupon\CreateAndEditViewModel;
use App\ViewModels\CMS\Coupon\IndexViewModel;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\Validator;

class CouponController extends Controller
{
    public function index(Request $request)
    {
        Gate::authorize('couponViewAny', Coupon::class);

        $type = $request->tab ?? "manual";

        $coupons = Coupon::query()
            ->where("type", $type)
            ->withCount([
                "userRelationships" => function ($query) {
                    $query->where("coupon_user_relationships.status", "used");
                }
            ])
            ->orderBy("id", "desc")
            ->paginate(10);

        // dd($coupons->items()[0]->toArray());


        $viewModel = new IndexViewModel($request, $coupons, $request->tab);
        return view('cms.coupon.index', $viewModel);
    }

    public function create(Request $request)
    {
        Gate::authorize("couponCreate", Coupon::class);

        $viewModel = new  CreateAndEditViewModel($request, null, null);
        return view('cms.coupon.create', $viewModel);
    }

    public function store(Request $request)
    {
        Gate::authorize("couponCreate", Coupon::class);

        Validator::make($request->all(), [
            'code' => 'required|unique:coupons',
            'title' => 'required',
            'description' => 'nullable',
            'status' => 'required',
            'quantity' => 'nullable|integer',
            'max_use_per_order' => 'nullable|integer',
            'min_spend' => 'nullable|integer',
            'userNumber' => 'nullable',
            'discount_type' => 'required',
            'discount_value' => 'required|numeric',
            'start_at' => 'required|date',
            'end_at' => 'required|date|after:start_at',
        ])->validate();


        // 這裡應該要整合到 CouponRepository 避免散亂
        $coupon = new Coupon();
        $coupon->code = $request->code;
        $coupon->title = $request->title;
        $coupon->description = $request->description;
        $coupon->status = $request->status;
        $coupon->quantity = $request->quantity;
        $coupon->max_use_per_order = $request->max_use_per_order ?? 1;
        $coupon->min_spend = $request->min_spend ?? 0;
        $coupon->discount_type = $request->discount_type;
        $coupon->discount_value = $request->discount_value;
        $coupon->start_at = $request->start_at;
        $coupon->end_at = $request->end_at;
        $coupon->type = "manual";

        if (isset($request->userNumber) and $request->status === "user") {

            // 確認會員存在嗎
            $user = UserRepository::getUserByNumber($request->userNumber);
            if ($user === null) {
                return redirect()->back()->with(Status::bladeFails("指定的會員不存在"));
            }
        }


        $coupon->save();

        // 如果是綁定會員，那麼就要新增一筆 coupon_user_relationships
        if (isset($request->userNumber) and $request->status === "user") {
            $couponUserRelationship = CouponUserRelationshipRepository::insert(
                $coupon->id,
                $user->id,
                "active", null,
                [
                    'source' => "cmsInsert",
                ]
            );
        }

        return redirect()->route('cms.coupon.index')
            ->with(Status::bladeSuccess("優惠券已新增成功"));
    }


    public function edit(Request $request, Coupon $coupon)
    {
        Gate::authorize("couponUpdate", Coupon::class);

        if ($coupon->status === "user") {
            $coupon = Coupon::query()->where("id", $coupon->id)
                ->with("users")
                ->first();
        }

        $viewModel = new  CreateAndEditViewModel(
            $request,
            $coupon,
            null
        );

        return view('cms.coupon.create', $viewModel);
    }

    public function update(Request $request, Coupon $coupon)
    {
        Gate::authorize("couponUpdate", Coupon::class);

        Validator::make($request->all(), [
            'code' => 'required|unique:coupons,code,'.$coupon->id,
            'discount_type' => 'required',
            'discount_value' => 'required',
            'title' => 'required',
            'description' => 'nullable',
            'start_at' => 'required|date',
            'end_at' => 'required|date|after:start_at',
        ], [
            'code.required' => '請輸入優惠碼',
            'code.unique' => '優惠碼已存在',
            'discount_type.required' => '請選擇折扣類型',
            'discount_value.required' => '請輸入折扣值',
            'title.required' => '請輸入標題',
            'start_at.required' => '請輸入開始時間',
            'end_at.required' => '請輸入結束時間',
            'end_at.after' => '截止時間必須大於開始時間',
        ])->validate();

        $coupon->code = $request->code;
        $coupon->discount_type = $request->discount_type;
        $coupon->discount_value = $request->discount_value;
        $coupon->title = $request->title;
        $coupon->description = $request->description;
        $coupon->start_at = $request->start_at;
        $coupon->end_at = $request->end_at;
        $coupon->status = $request->status;
        $coupon->save();

        return redirect()->route('cms.coupon.index')
            ->with(Status::bladeSuccess("優惠券已更新成功"));
    }

    public function destroy(Coupon $coupon)
    {
        Gate::authorize("couponDelete", Coupon::class);

        $coupon->delete();

        return redirect()->route('cms.coupon.index')
            ->with('success', '優惠券已刪除成功！');
    }

    /**
     * 強制清除前端所有快娶
     */
    public function allCacheDestroy()
    {
        Gate::authorize("couponDelete", Coupon::class);

        Cache::flush();
        return redirect()->back()->with(Status::bladeSuccess("快取已清除"));
    }
}
