<?php

namespace App\Http\Controllers\CMS;

use App\Enums\PostImageSize;
use App\Http\Controllers\Controller;
use App\Models\Image;
use App\Libraries\ErrorMessage;
use App\Libraries\Status;
use App\Models\Post;
use App\Repositories\ImageRepository;
use App\Repositories\PostCategoryRepository;
use App\Repositories\PostRepository;
use App\Services\CMS\PostCategoryService;
use App\Services\CMS\PostService;
use App\Services\Common\ImageService;
use App\ViewModels\CMS\Knowledge\CreateAndEditViwModel;
use App\ViewModels\CMS\Knowledge\Image\EditViewModel;
use App\ViewModels\CMS\Knowledge\IndexViewModel;
use App\ViewModels\CMS\TemplateViewModel;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Storage;
use Illuminate\View\View;
use PhpParser\Node\Stmt\DeclareDeclare;

/**
 * 知識學堂，採用 Post 服務
 */
class KnowledgeController extends Controller
{
    /**
     * 顯示知識學堂列表
     *
     * @param  Request  $request
     *
     * @return Application|Factory|View
     */
    public function index(Request $request)
    {
        Gate::authorize("knowledgeViewAny", Post::class);

        $categoryIds = PostCategoryService::getRootCategoriesByParentId(200, $request->categoryId, ['id', 'title']);

        $posts = PostService::getPosts2($categoryIds);

        $postCategories = PostCategoryRepository::getCategoriesByParentId(
            200,
            [
                'id',
                'title'
            ]
        );

        $indexViewModel = new  IndexViewModel(
            $request,
            $postCategories,
            $posts,
            $request->tab
        );

        return view("cms.knowledge.index", $indexViewModel);
    }

    /**
     * 建立知識學堂文章
     *
     * @param  Request  $request
     *
     * @return Application|Factory|View
     */
    public function create(Request $request)
    {
        Gate::authorize("knowledgeCreate", Post::class);

        $postCategories = PostCategoryRepository::getCategoriesByParentId(
            200,
            [
                'id',
                'title'
            ]
        );

        $createAndEditViwModel = new  CreateAndEditViwModel(
            $request,
            "create",
            $postCategories,
            null,
            null
        );

        return view("cms.knowledge.create", $createAndEditViwModel);
    }

    /**
     * 新增知識學堂
     *
     * @param  Request  $request
     *
     * @return Application|Factory|View
     */
    public function store(Request $request)
    {
        Gate::authorize("knowledgeCreate", Post::class);

        // 若有上傳封面，取得編號
        if ($request->hasFile('cover')) {
            $result = ImageService::insert(
                Auth::guard("cms")->user()->id,
                $request->file('cover'),
                1, false
            );

            $coverId = $result['data']['image']['id'];
        } else {
            $coverId = null;
        }

        $result = PostService::insert(
            $request->status,
            $coverId,
            $request->title,
            $request['content'],
            $request->summary,
            $request->publishedAt,
            null,
            Auth::guard("cms")->id(),
            $request->categories
        );

        return $result;
    }

    /**
     * 編輯知識學堂文章
     *
     * @param  Request  $request
     * @param  string  $id
     *
     * @return Application|Factory|View
     */
    public function edit(Request $request, string $id)
    {
        Gate::authorize("knowledgeUpdate", Post::class);

        $result = PostService::getPost($id, "images");
        if (Status::isFails($result)) {
            abort(404);
        }

        $postCategories = PostCategoryRepository::getCategoriesByParentId(
            200,
            [
                'id',
                'title'
            ]
        );

        $editViewModel = new CreateAndEditViwModel(
            $request,
            "edit",
            $postCategories,
            $result['data']['post'],
            null
        );

        return view("cms.knowledge.create", $editViewModel);
    }

    /**
     * 圖片編輯頁面
     *
     * @param  Request  $request
     * @param  string  $id
     *
     * @return Application|Factory|View
     */
    public function imageEdit(Request $request, $id)
    {
        Gate::authorize("knowledgeUpdate", Post::class);

        $result = ImageService::getImage($id);
        if (Status::isFails($result)) {
            abort(403, "這張圖片已經失效");
        }

        $editViewModel = new EditViewModel($request, $id, $result['data']['image']);
        return view("cms.knowledge.imageEdit", $editViewModel);
    }

    /**
     * 刪除知識學堂文章
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function destroy(Request $request)
    {
        Gate::authorize("knowledgeDelete", Post::class);

        $result = PostService::delete($request->id);
        if ($result === false) {
            return redirect()->back()->with(Status::bladeFails("您沒有刪除任何一筆資料"));
        }
        return redirect()->back()->with(Status::bladeSuccess("刪除成功"));
    }

    /**
     * 刪除知識學堂的畫廊圖片
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageDestroy(Request $request)
    {
        Gate::authorize("knowledgeDelete", Post::class);

        $deleteById = ImageService::deleteById($request->id, null);
        return redirect()->back()->with(Status::bladeSuccess("刪除成功"));
    }

    /**
     * 使用網址夾帶訊息，導向指定的頁面
     * @param  Request  $request
     * @param  string  $message
     *
     * @return RedirectResponse
     */
    public function redirectMessageIndex(Request $request, $message)
    {
        if (isset($request->redirect)) {
            return redirect($request->redirect)->with(
                Status::bladeSuccess($message)
            );
        }

        return redirect()->route("cms.knowledge.index")->with(
            Status::bladeSuccess($message)
        );
    }

    /**
     * 更新知識學堂文章
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function update(Request $request)
    {
        Gate::authorize("knowledgeUpdate", Post::class);

        $result = PostService::update(
            $request,
            $request->id,
            $request->status,
            $request->coverId,
            $request->title,
            $request['content'],
            $request->summary,
            $request->publishedAt,
            null,
            Auth::guard("cms")->id(),
            $request->postCategoryIds
        );

        return $result;
    }

    /**
     * 更新知識學堂的畫廊排序
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function gallerySortUpdate(Request $request)
    {
        Gate::authorize("knowledgeUpdate", Post::class);

        $result = PostService::updateGallerySort($request->postId, $request->imageSort);
        return $result;
    }

    /**
     * 更新知識學堂的畫廊圖片
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageUpdate(Request $request)
    {
        Gate::authorize("knowledgeUpdate", Post::class);

        $result = PostService::updateImage($request->id, $request->title, null, "", "", "", null, null);

        return redirect()->away(urldecode($request->redirect))->with(
            Status::bladeSuccess("更新成功")
        );
    }

    /**
     * 上傳知識學堂文章的畫廊圖片
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageStore(Request $request)
    {
        Gate::authorize("knowledgeCreate", Post::class);

        $images = $request->file('images');
        $result = PostService::insertImages($request->id, $images, PostImageSize::Desktop);
        return $result;
    }
}
