<?php

namespace App\Http\Controllers\CMS;

use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\Order;
use App\Repositories\OrderRepository;
use App\Services\CMS\OrderService;
use App\Services\CMS\LogisticsService;
use App\ViewModels\CMS\Order\EditViewModel;
use App\ViewModels\CMS\Order\IndexViewModel;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;

class OrderController extends Controller
{
    public function index(Request $request)
    {
        Gate::authorize("orderViewAny", Order::class);
//        dd($request);
        $orders = OrderService::getOrders(
            $request->userNumber,
            $request->orderNumber,
            $request->createAtFrom,
            $request->createAtTo,
            $request->productId,
            $request->couponId
        );

        $viewModel = new  IndexViewModel(
            $request,
            $orders,
            $request->tab
        );
        return view("cms.order.index", $viewModel);
    }

    /**
     * @throws Exception
     */
    public function indexExport(Request $request)
    {
        Gate::authorize("orderViewAny", Order::class);

        $result = OrderService::getOrdersExport(
            $request->userNumber,
            $request->orderNumber,
            $request->productId,
            $request->createAtFrom,
            $request->createAtTo,
            $request->couponId
        );

        if (Status::isFails($result)) {
            return redirect()->back()
                ->with(Status::bladeFails($result['message']));
        }

        return $result['data']['streamedResponse'];
    }

    public function edit(Request $request, string $orderNo)
    {
        Gate::authorize("orderUpdate", Order::class);

        Validator::make([
            'orderNo' => $orderNo,
            'logisticSettings' => $request->logisticSettings,
        ], [
            'orderNo' => 'required',
            'logisticSettings' => 'nullable',
        ])->validate();

        $result = OrderService::getOrderByOrderNumber($orderNo);

        $order = $result['data']['order'];

        // $logisticsStatus = LogisticsService::getOrder($order);dd($logisticsStatus);

        $viewModel = new EditViewModel(
            $request,
            $order
        );
        return view("cms.order.edit", $viewModel);
    }

    /**
     * TODO: 或許這裡放到 web.php 底下的 LogisticsController 集中會比較好，等有空再回台調整優化。這裡應該拆成兩半段。上半段判斷訂單所記錄的物流資訊，確認無誤以後，再送到 LogisticsController 列印
     */
    public function printLogisticsShow(Request $request, Order $order)
    {
        Gate::authorize("orderViewAny", Order::class);

        Validator::make($request->all(), [
            'redirect' => 'required|url',
            'goodsAmount' => 'required|numeric',
            'specification' => 'required',
            'remark' => 'nullable',
        ])->validate();


        // 如果已經建立物流正式單，就直接列印
        if (($order->logistics_raw_data_formal['RtnCode'] ?? null) !== 1) {
            Log::info("修改暫存物流單", compact('request'));

            $resultInsert = LogisticsService::insertFormalOrder(
                $order,
                $request->goodsAmount,
                $request->specification,
                $request->remark ?? ""
            );


            Log::info("進行建立物流正式單", compact('resultInsert'));
            if (Status::isFails($resultInsert)) {
                return redirect()->back()
                    ->with(Status::bladeFails($resultInsert['message']));
            }
        }

        $resultPrint = LogisticsService::getPrint($order);

        Log::info("進行列印物流單", compact('resultPrint'));


        return $resultPrint;
    }


    public function update(Request $request, Order $order)
    {
        Gate::authorize("orderUpdate", Order::class);

        Validator::make($request->all(), [
            'paymentStatus' => 'required',
            'shippingStatus' => 'required',
            'redirect' => 'required|url',
        ])->validate();

        $order->payment_status = $request->paymentStatus;
        $order->shipping_status = $request->shippingStatus;
        $order->save();


        return redirect()->away($request->redirect)
            ->with(Status::bladeSuccess("訂單更新成功"));
    }

    public function destroy(Order $order)
    {
        Gate::authorize("orderDelete", Order::class);

        $order->delete();
        return redirect()->back()
            ->with(Status::bladeSuccess("訂單刪除成功"));
    }
}
