<?php

namespace App\Http\Controllers\CMS;

use App\Models\ProductCategory;
use App\Repositories\PostCategoryRepository;
use App\Repositories\ProductCategoryRepository;
use App\Services\CMS\PostCategoryService;
use App\Services\CMS\ProductCategoryService;
use App\Services\CMS\ProductService;
use App\Services\Common\ImageService;
use App\ViewModels\CMS\ProductCategory\CreateViewModel;
use App\ViewModels\CMS\ProductCategory\EditViewModel;
use App\ViewModels\CMS\ProductCategory\IndexViewModel;
use App\Http\Controllers\Controller;
use App\Libraries\Status;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\Validator;

class ProductCategoryController extends Controller
{
    public function index(Request $request)
    {
        Gate::authorize("productCategoryViewAny", ProductCategory::class);

        Validator::make([
            // 搜尋的某個分類 id 底下的所有類別
            'searchId' => $request->searchId,
        ], [
            'searchId' => 'nullable|integer|string'
        ])->validate();

        // 設定 breadcrumbs 的 view
        Config::set('breadcrumbs.view', 'cms/layouts/breadcrumbs');

        $page = ProductCategoryService::getPageIndex(
            rootProductCategoryId: null,
            searchId: $request->searchId
        );

        if (Status::isFails($page)) {
            return redirect()->route("cms.productCategory.index")->with(
                Status::bladeFails($page['message'])
            );
        }

        $indexViewModel = new IndexViewModel(
            $request,
            $page['data']['descendents'],
            $page['data']['ancestors'],
            $page['data']['searchProductCategory'],
            $request->tab
        );
        return view("cms.product-category.index", $indexViewModel);
    }

    public function create(Request $request)
    {
        Gate::authorize("productCategoryCreate", ProductCategory::class);

        Validator::make([
            'parentId' => $request->parentId,
        ], [
            'searchId' => 'nullable|integer|string'
        ])->validate();

        // 如果是新增下層分類
        if (!empty($request->parentId)) {
            $parentProductCategory = ProductCategoryService::getProductCategory($request->parentId);
            if (Status::isFails($parentProductCategory)) {
                return redirect()->route("cms.productCategory.index")->with(
                    Status::bladeFails("找不到您指定的上層分類，可能已經不存在")
                );
            }
        }

        // 如果沒有指定上層分類，那就是新增預設分類
        $parentId = $request->parentId ?? null;

        $createViewModel = new CreateViewModel(
            $request,
            $parentId,
            $parentProductCategory['data']['productCategory'] ?? null
        );

        return view("cms.product-category.create", $createViewModel);
    }

    public function store(Request $request)
    {
        Gate::authorize("productCategoryCreate", ProductCategory::class);

        Validator::make([
            'title' => $request->title,
            'parentId' => $request->parentId,
            'cover' => $request->cover,
        ], [
            'title' => 'required|string|max:255',
            'parentId' => 'nullable|integer|string',
            'cover' => 'nullable|image',
        ])->validate();


        $result = ProductCategoryService::insert(
            $request->title,
            $request->parentId,
            $request->cover
        );

        if (Status::isFails($result)) {
            return redirect()->back()
                ->with(Status::bladeFails($result['message']));
        }

        return redirect()->away($request->redirect ?? route("cms.productCategory.index"))
            ->with(Status::bladeSuccess("新增成功"));
    }

    public function edit(Request $request, string $productCategoryId)
    {
        Gate::authorize("productCategoryUpdate", ProductCategory::class);

        $result = ProductCategoryService::getProductCategory($productCategoryId);

        if (Status::isFails($result)) {
            return redirect()->route("cms.productCategory.index")->with(
                Status::bladeFails($result['message'])
            );
        }

        $productCategoryViewModel = new  EditViewModel(
            $request,
            $result['data']['productCategory'],
            null
        );

        return view("cms.product-category.edit", $productCategoryViewModel);
    }

    public function update(Request $request)
    {
        Gate::authorize("productCategoryUpdate", ProductCategory::class);

        $result = ProductCategoryService::update(
            $request->title,
            $request->id,
            $request->cover
        );

        if (Status::isFails($result)) {
            return redirect()->back()
                ->with(Status::bladeFails($result['message']));
        }

        return redirect()->away($request->redirect ?? route("cms.productCategory.index"))
            ->with(Status::bladeSuccess("修改成功"));
    }

    public function destroy(Request $request)
    {
        Gate::authorize("productCategoryDelete", ProductCategory::class);

        // 如果底下有後代分類，不予刪除
        $result = ProductCategoryService::getDescendentsByProductCategoryId($request->id);
        if (Status::isSuccess($result)) {
            return redirect()->away($request->redirect ?? route("cms.productCategory.index"))
                ->with(Status::bladeFails("底下有後代分類，因此無法刪除"));
        }

        // 如果底下有產品，不予刪除
        $productResult = ProductService::getProductsByProductCategoryId($request->id);
        if (Status::isSuccess($productResult)) {
            return redirect()->route("cms.productCategory.index")
                ->with(Status::bladeFails("底下有產品，因此無法刪除"));
        }

        $result = ProductCategoryService::delete($request->id);

        if (Status::isFails($result)) {
            return redirect()->route("cms.productCategory.index")
                ->with(Status::bladeFails($result['message']));
        }

        return redirect()->away($request->redirect ?? route("cms.productCategory.index"))
            ->with(Status::bladeSuccess("分類刪除成功"));
    }
}
