<?php

namespace App\Http\Controllers\CMS;

use App\Http\Controllers\Controller;
use App\Libraries\Text;
use App\Models\Image;
use App\Libraries\ErrorMessage;
use App\Libraries\Status;
use App\Models\Product;
use App\Repositories\ImageRepository;
use App\Repositories\PostCategoryRepository;
use App\Repositories\PostRepository;
use App\Repositories\ProductRepository;
use App\Services\CMS\PostCategoryService;
use App\Services\CMS\PostService;
use App\Services\CMS\ProductCategoryRelationshipService;
use App\Services\CMS\ProductCategoryService;
use App\Services\CMS\ProductService;
use App\Services\CMS\ProductSpecialPromotionService;
use App\Services\CMS\ProductVariantService;
use App\Services\Common\ImageService;
use App\ViewModels\CMS\Product\CreateAndEditViwModel;
use App\ViewModels\CMS\Product\Image\EditViewModel;
use App\ViewModels\CMS\Product\IndexViewModel;
use App\ViewModels\CMS\TemplateViewModel;
use Exception;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Illuminate\View\View;
use PhpParser\Node\Stmt\DeclareDeclare;

/**
 * 產品，採用 Post 服務
 */
class ProductController extends Controller
{
    /**
     * 顯示產品列表
     *
     * @param  Request  $request
     *
     * @return Application|Factory|RedirectResponse|View
     * @throws Exception
     */
    public function index(Request $request)
    {
        Gate::authorize("productViewAny", Product::class);

        Validator::make(
            $request->all(),
            [
                'productCategories' => 'nullable',
                'tab' => 'nullable',
                'keyword' => 'nullable',
            ]
        )->validate();

        // // 若正在設計如一頁式網站：因為沒有分類需要，但是為了使用排序，如果沒有指定分類，則導向第一個分類的第一個 descendent
        // if ($this->isProductCategoriesExistsInRequest($request)) {
        //     $setProductCategories = $this->getRedirectCategoriesWhenEmpty();
        //     return redirect()->route("cms.product.index", $setProductCategories);
        // }


        // 產品分類後代
        $productCategories = ProductCategoryService::getDescendentsByProductCategoryId(null)['data']['descendents'];

        // 指定分類底下的最後一層分類的所有產品
        $products = ProductService::getProducts($request->productCategories ?? [], $request->keyword);

        $indexViewModel = new  IndexViewModel(
            $request,
            $productCategories,
            $products,
            $request->keyword,
            $request->tab
        );

        return view("cms.product.index", $indexViewModel);
    }

    /**
     * 建立產品
     *
     * @param  Request  $request
     *
     * @return Application|Factory|View
     */
    public function create(Request $request)
    {
        Gate::authorize("productCreate", Product::class);

        $productCategories = ProductCategoryService::getDescendentsByProductCategoryId(null)['data']['descendents'];

        // 限制不顯示的分類。務必使用 values 重新排序，避免 JS 無法 parse 給 each 使用
        $productCategories = $productCategories->whereNotIn('id', [1000])->values();
        $checkProductStatus = ProductService::getProductsByType('addOn',null);
        if($checkProductStatus['message'] ==="無任何種類"){
            $productsAddOn = null;
        }else{
            $productsAddOn = ProductService::getProductsByType('addOn',null)['data']['products'];
        }

        // dd($checkProductStatus);


        $createAndEditViwModel = new  CreateAndEditViwModel(
            $request,
            "create",
            $productCategories,
            $productsAddOn,
            null,
            null,
            null
        );
//        dd($createAndEditViwModel);
        return view("cms.product.create", $createAndEditViwModel);
    }

    /**
     * 新增產品
     *
     * @param  Request  $request
     * @return array|JsonResponse
     */
    public function store(Request $request)
    {
        Gate::authorize("productCreate", Product::class);

        // 驗證太多了，這裡只做簡單的防呆
        $result = Validator::make(
            $request->all(),
            [
                'title' => 'required',
                'categories' => 'array',
                'startAt' => 'required',
                'endAt' => 'required|after:startAt',
                'chooseSpecifyShippingFee.storePickup' => [
                    'nullable',
                    function ($attribute, $value, $fail) use ($request) {
                        if (isset($value) && (!isset($request->specifyShippingFee['storePickup']) || !ctype_digit($request->specifyShippingFee['storePickup']) || $request->specifyShippingFee['storePickup'] <= 0)) {
                            $fail('超商取貨必須填寫金額');
                        }
                    },
                ],
                'chooseSpecifyShippingFee.homeDelivery' => [
                    'nullable',
                    function ($attribute, $value, $fail) use ($request) {
                        if (isset($value) && (!isset($request->specifyShippingFee['homeDelivery']) || !ctype_digit($request->specifyShippingFee['homeDelivery']) || $request->specifyShippingFee['homeDelivery'] <= 0)) {
                            $fail('宅配必須填寫金額');
                        }
                    },
                ],
            ],
            [
                'title.required' => '請輸入產品名稱',
                'categories.array' => '請選擇產品分類',
                'startAt.required' => '請輸入開始販售時間',
                'endAt.required' => '請輸入結束販售時間',
                'endAt.after' => '結束時間必須在開始時間之後',
            ]
        );

        if ($result->fails()) {
            return response()->json(Status::fails(
                "驗證錯誤",
                "validate",
                $result->errors()->all()
            ), 400);
        }
//        dd($request);
        $covers = $this->upload($request);
        if($request->addOn) {
            $request->type = "addOn";
        }

        $result = ProductService::insert(
            $request->title,
            $request->type,
            $request->categories,
            $request->temperature,
            $request->description,
            $request->attentionNote,
            $request->summary,
            $request->tags,
            $request->status,
            $request->startAt,
            $request->endAt,

            $request->variantTitles,
            $request->attributeNumbers,

            $request->attributeTitles,
            $request->attributeSalePrices,
            $request->attributePrices,
            $request->attributeStockQuantities,
            $request->attributeStatus,
            $request->attributeDiscountPrices,
            $request->attributeDiscounts,
            $covers,
            $request->chooseSpecifyShippingFee ?? [],
            $request->specifyShippingFee,
        );
        //要取得新的一筆新增的資料的ＩＤ才可以記到加購資料表
//        dd($request->addOnInput);
        ProductService::getLatestProductAndInsertAddOn($request->addOnInput);
        $product = $result['data']['product'];
//        dd($product);
        return Status::success("成功新增產品", "success", compact('product'));
    }

    /**
     * 複製產品
     * @param  Request  $request
     * @param  Product  $product
     * @return RedirectResponse
     */
    public function replicateStore(Request $request, Product $product)
    {
        $result = ProductService::replicate($product);

        if (Status::isFails($result)) {
            return redirect()->back()
                ->with(Status::bladeFails("複製失敗"));
        }

        return redirect()->route("cms.product.edit",
            ['id' => $result['data']['productId'], "redirect" => $request->redirect])
            ->with(Status::bladeSuccess("複製成功，狀態已更新為下架，請確認您的新商品。"));
    }

    /**
     * 編輯產品
     *
     * @param  Request  $request
     * @param  string  $id
     *
     * @return Application|Factory|View
     */
    public function edit(Request $request, string $id)
    {
        Gate::authorize("productUpdate", Product::class);

        $result = ProductService::getProductById($id);

        if (Status::isFails($result)) {
            abort(404);
        }

        // 產品分類後代
        $productCategories
            = ProductCategoryService::getDescendentsByProductCategoryId(null)['data']['descendents'];

        $checkProductStatus = ProductService::getProductsByType('addOn', $id);

        // 取出全部的加購產品
        if($checkProductStatus['message'] ==="無任何種類"){
            $productsAddOn = null;
            $getProductsAddOnByProductId = null;
        }else{
            $productsAddOn = ProductService::getProductsByType('addOn',$id)['data']['products'];
            // select productsAddon where 這項產品的id -> 轉換為陣列
            $getProductsAddOnByProductId = ProductService::getProductsAddOnByProductId($result["data"]["product"]["id"]);
        }


        $editViewModel = new CreateAndEditViwModel(
            $request,
            "edit",
            $productCategories,
            $productsAddOn,
            $result['data']['product'],
            null,
            $getProductsAddOnByProductId,
        );
        //        dd($editViewModel);
        return view("cms.product.create", $editViewModel);
    }

    /**
     * 圖片編輯頁面
     *
     * @param  Request  $request
     * @param  string  $id
     *
     * @return Application|Factory|View
     */
    public function imageEdit(Request $request, $id)
    {
        Gate::authorize("productUpdate", Product::class);

        $result = ImageService::getImage($id);
        if (Status::isFails($result)) {
            abort(403, "這張圖片已經失效");
        }

        $editViewModel = new EditViewModel($request, $id,
            $result['data']['image']);
        return view("cms.product.imageEdit", $editViewModel);
    }

    /**
     * 刪除產品
     * 因為來源暫時無法改寫成 form with Delete method，所以只好用 get
     *
     * @param  Request  $request
     * @param  Product  $product
     * @return RedirectResponse
     */
    public function destroy(Request $request, Product $product)
    {
        Gate::authorize("productDelete", Product::class);

        $result = ProductService::delete($product->id);

        if (Status::isFails($result)) {
            return redirect()->back()
                ->with(Status::bladeFails("您沒有刪除任何一筆資料"));
        }
        return redirect()->back()->with(Status::bladeSuccess("刪除成功"));
    }

    /**
     * 刪除產品的畫廊圖片
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageDestroy(Request $request)
    {
        Gate::authorize("productDelete", Product::class);

        $deleteById = ImageService::deleteById($request->id, null);
        return redirect()->back()->with(Status::bladeSuccess("刪除成功"));
    }

    /**
     * 使用網址夾帶訊息，導向指定的頁面
     *
     * @param  Request  $request
     * @param  string  $message
     *
     * @return RedirectResponse
     */
    public function redirectMessageIndex(Request $request, $message)
    {
        if (isset($request->redirect)) {
            return redirect($request->redirect)->with(
                Status::bladeSuccess($message)
            );
        }

        return redirect()->route("cms.product.index")->with(
            Status::bladeSuccess($message)
        );
    }

    /**
     * 更新產品
     *
     * @param  Request  $request
     * @return array
     */
    public function update(Request $request)
    {
        Gate::authorize("productUpdate", Product::class);

        $newCoverIds = $this->upload($request);

        if ($request->addOn){
            $request->type = "addOn";
        }else{
            $request->type = "product";
        }


        $result = ProductService::update(
            $request->id,
            $request->title,
            $request->type,
            $request->temperature,
            $request->categories,
            $request->description,
            $request->attentionNote ?? "",
            $request->summary ?? "",
            $request->tags ?? "",
            $request->status,
            $request->startAt,
            $request->endAt,

            $request->variantIds,
            $request->variantTitles,
            $request->attributeId,
            $request->attributeNumbers,
            $request->attributeTitles,
            $request->attributeSalePrices,
            $request->attributePrices,
            $request->attributeStockQuantities,
            $request->attributeStatus,
            $request->attributeDiscountPrices,
            $request->attributeDiscounts,
            $request->attributeCoverIds,
            $newCoverIds,
            $request->chooseSpecifyShippingFee ?? [],
            $request->specifyShippingFee,
        );

        ProductRepository::deleteProductsAddOnByProductId($request->id);
        if($request->addOnInput && $request->type = "product"){
            ProductRepository::getLatestProductAndInsertAddOn($request->addOnInput,$request->id);
        }

        return $result;
    }

    /**
     * 更新產品的畫廊排序
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function gallerySortUpdate(Request $request)
    {
        Gate::authorize("productUpdate", Product::class);

        $result = ProductService::updateGallerySort($request->productId,
            $request->imageSort);
        return $result;
    }

    /**
     * 更新產品的畫廊圖片
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageUpdate(Request $request)
    {
        Gate::authorize("productUpdate", Product::class);

        ProductService::updateImage($request->id, $request->title);

        return redirect()->away(urldecode($request->redirect))->with(
            Status::bladeSuccess("更新成功")
        );
    }

    /**
     * 上傳產品的畫廊圖片
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageStore(Request $request)
    {
        Gate::authorize("productCreate", Product::class);

        $images = $request->file('images');
        $result = ProductService::insertImages($request->id, $images);
        return $result;
   }

    /**
     * @param  Request  $request
     * @return array
     */
    public function upload(Request $request): array
    {
        Gate::authorize("productCreate", Product::class);

        $covers = [];
        if (is_array($request->attributeCovers)) {
            foreach ($request->attributeCovers as $variantIndex => $attributeItems) {
                foreach ($attributeItems as $attributeIndex => $coverFile) {
                    $result = ImageService::insert(
                        Auth::guard("cms")->user()->id,
                        $coverFile,
                        1, false
                    );
                    $covers[$variantIndex][$attributeIndex] = $result['data']['image']['id'];
                }
            }
        }
        return $covers;
    }

    public function specialPromotionStore(Request $request)
    {
        Gate::authorize("productCreate", Product::class);

        $result = ProductSpecialPromotionService::insert(
            $request->productIds ?? [],
        );
        return $result;
    }

    public function specialPromotionDestroy(Request $request)
    {
        Gate::authorize("productDelete", Product::class);

        $result = ProductSpecialPromotionService::delete(
            $request->productIds ?? [],
        );
        return $result;
    }

    public function sortUpdate(Request $request)
    {
        Gate::authorize("productUpdate", Product::class);

        $result = ProductService::updateSort($request->productSort);
        return $result;
    }

    /**
     * 取得產品分類的第一個項目與第一個 descendent
     *
     * 億來：因為沒有分類需要，但是為了使用排序，如果沒有指定分類，則導向第一個分類的第一個 descendent
     *
     * @return array
     * @throws Exception
     */
    private function getRedirectCategoriesWhenEmpty(): array
    {
        $productCategories = ProductCategoryService::getDescendentsByProductCategoryId(null);
        if (ErrorMessage::isExists($productCategories)) {
            throw new Exception($productCategories['message']);
        }


        $setProductCategories = [];
        $setProductCategories['productCategories'][0] = $productCategories['data']['descendents'][0]['id'];
        $setProductCategories['productCategories'][1] = $productCategories['data']['descendents'][0]['descendants'][0]['id'];
        return $setProductCategories;
    }

    /**
     * 判斷是否有產品分類存在於 Request
     * @param  Request  $request
     * @return bool
     */
    private function isProductCategoriesExistsInRequest(Request $request): bool
    {
        return $request->productCategories === null or $request->productCategories[0] === null;
    }
}
