<?php

namespace App\Http\Controllers\CMS;

use App\Enums\PostImageSize;
use App\Http\Controllers\Controller;
use App\Models\Image;
use App\Libraries\ErrorMessage;
use App\Libraries\Status;
use App\Models\Post;
use App\Models\PostStore;
use App\Repositories\ImageRepository;
use App\Repositories\PostCategoryRepository;
use App\Repositories\PostRepository;
use App\Services\CMS\PostCategoryService;
use App\Services\CMS\PostService;
use App\Services\CMS\PostStoreService;
use App\Services\Common\ImageService;
use App\ViewModels\CMS\Store\CreateAndEditViwModel;
use App\ViewModels\CMS\Store\Image\EditViewModel;
use App\ViewModels\CMS\Store\IndexViewModel;
use App\ViewModels\CMS\TemplateViewModel;
use Exception;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Gate;
use Illuminate\Support\Facades\Redirect;
use Illuminate\Support\Facades\Storage;
use Illuminate\View\View;
use PhpParser\Node\Stmt\DeclareDeclare;

/**
 * 店面，採用 Post 服務
 */
class StoreController extends Controller
{
    /**
     * 顯示店面列表
     *
     * @param  Request  $request
     *
     * @return Application|Factory|View
     */
    public function index(Request $request)
    {
        Gate::authorize("newsViewAny", Post::class);

        $categoryIds = PostCategoryService::getRootCategoriesByParentId(600, $request->categoryId, ['id', 'title']);

        PostStoreService::setRelation(true);
        $posts = PostService::getPosts2($categoryIds);

        $postCategories = PostCategoryRepository::getCategoriesByParentId(
            600,
            [
                'id',
                'title'
            ]
        );

        $indexViewModel = new  IndexViewModel(
            $request,
            $postCategories,
            $posts,
            $request->tab
        );

        return view("cms.store.index", $indexViewModel);
    }

    /**
     * 建立店面
     *
     * @param  Request  $request
     *
     * @return Application|Factory|View
     */
    public function create(Request $request)
    {
        Gate::authorize("newsCreate", Post::class);

        $postCategories = PostCategoryRepository::getCategoriesByParentId(
            600,
            [
                'id',
                'title'
            ]
        );

        $createAndEditViwModel = new  CreateAndEditViwModel(
            $request,
            "create",
            $postCategories,
            null,
            null
        );

        return view("cms.store.create", $createAndEditViwModel);
    }

    /**
     * 新增店面
     *
     * @param  Request  $request
     * @return array|JsonResponse
     */
    public function store(Request $request)
    {
        Gate::authorize("newsCreate", Post::class);

        // 若有上傳封面，取得編號
        if ($request->hasFile('cover')) {
            $result = ImageService::insert(
                Auth::guard("cms")->user()->id,
                $request->file('cover'),
                1, false
            );

            $coverId = $result['data']['image']['id'];
        } else {
            $coverId = null;
        }

        $result = PostService::insert(
            $request->status,
            $coverId,
            $request->title,
            $request['content'],
            $request->summary,
            $request->publishedAt,
            null,
            Auth::guard("cms")->id(),
            $request->categories
        );

        if (Status::isFails($result)) {
            return response()->json($result, 500);
        }

        $resultPostStore = PostStoreService::insertOrUpdateStore(
            postId: $result['data']['postId'],
            address: $request->address,
            phone: $request->phone,
            fax: $request->fax,
            businessHours: $request->businessHours,
            googleMap: $request->googleMap,
        );

        if (Status::isFails($resultPostStore)) {
            return response()->json($resultPostStore, 500);
        }

        return $result;
    }

    /**
     * 編輯店面
     *
     * @param  Request  $request
     * @param  string  $id
     *
     * @return Application|Factory|View
     */
    public function edit(Request $request, string $id)
    {
        Gate::authorize("newsUpdate", Post::class);

        $result = PostService::getPost($id, "images");
        if (Status::isFails($result)) {
            abort(404);
        }

        // 擴充店家資訊
        $post = PostStoreService::extPostStore($result['data']['post']);


        $postCategories = PostCategoryRepository::getCategoriesByParentId(
            600,
            [
                'id',
                'title'
            ]
        );

        $editViewModel = new CreateAndEditViwModel(
            $request,
            "edit",
            $postCategories,
            $result['data']['post'],
            null
        );

        return view("cms.store.create", $editViewModel);
    }

    /**
     * 圖片編輯頁面
     *
     * @param  Request  $request
     * @param  string  $id
     *
     * @return Application|Factory|View
     */
    public function imageEdit(Request $request, $id)
    {
        Gate::authorize("newsUpdate", Post::class);

        $result = ImageService::getImage($id);
        if (Status::isFails($result)) {
            abort(403, "這張圖片已經失效");
        }

        $editViewModel = new EditViewModel($request, $id, $result['data']['image']);
        return view("cms.store.imageEdit", $editViewModel);
    }

    /**
     * 刪除店面
     * @param  Request  $request
     *
     * @return RedirectResponse
     * @throws Exception
     */
    public function destroy(Request $request)
    {
        Gate::authorize("newsDelete", Post::class);

        // 先刪除關聯表
        PostStoreService::delete($request->id);

        // 再刪除 post 表
        $result = PostService::delete($request->id);
        if ($result === false) {
            return redirect()->back()->with(Status::bladeFails("您沒有刪除任何一筆資料"));
        }

        return redirect()->back()->with(Status::bladeSuccess("刪除成功"));
    }

    /**
     * 刪除店面的畫廊圖片
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageDestroy(Request $request)
    {
        Gate::authorize("newsDelete", Post::class);

        $deleteById = ImageService::deleteById($request->id, null);
        return redirect()->back()->with(Status::bladeSuccess("刪除成功"));
    }

    /**
     * 使用網址夾帶訊息，導向指定的頁面
     * @param  Request  $request
     * @param  string  $message
     *
     * @return RedirectResponse
     */
    public function redirectMessageIndex(Request $request, $message)
    {
        if (isset($request->redirect)) {
            return redirect($request->redirect)->with(
                Status::bladeSuccess($message)
            );
        }

        return redirect()->route("cms.store.index")->with(
            Status::bladeSuccess($message)
        );
    }

    /**
     * 更新店面
     * @param  Request  $request
     *
     * @return array
     */
    public function update(Request $request)
    {
        Gate::authorize("newsUpdate", Post::class);

        $resultPost = PostService::update(
            $request,
            $request->id,
            $request->status,
            $request->coverId,
            $request->title,
            $request['content'],
            $request->summary,
            $request->publishedAt,
            null,
            Auth::guard("cms")->id(),
            $request->postCategoryIds
        );

        // 更新店家資訊
        $resultPostStore = PostStoreService::insertOrUpdateStore(
            postId: $request->id,
            address: $request->address,
            phone: $request->phone,
            fax: $request->fax,
            businessHours: $request->businessHours,
            googleMap: $request->googleMap,
        );

        return Status::success("店面更新成功", "success", compact('resultPost', 'resultPostStore'));
    }

    /**
     * 更新店面的畫廊排序
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function gallerySortUpdate(Request $request)
    {
        Gate::authorize("newsUpdate", Post::class);

        $result = PostService::updateGallerySort($request->postId, $request->imageSort);
        return $result;
    }

    /**
     * 更新店面的畫廊圖片
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageUpdate(Request $request)
    {
        Gate::authorize("newsUpdate", Post::class);

        $result = PostService::updateImage($request->id, $request->title, null, "", "", "", null, null);

        return redirect()->away(urldecode($request->redirect))->with(
            Status::bladeSuccess("更新成功")
        );
    }

    /**
     * 上傳店面的畫廊圖片
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function imageStore(Request $request)
    {
        Gate::authorize("newsCreate", Post::class);

        $images = $request->file('images');
        $result = PostService::insertImages($request->id, $images, PostImageSize::Desktop);
        return $result;
    }
}
