<?php

namespace App\Http\Controllers\Foreground;

use App\Enums\ShippingTemperatureForSystem;
use App\Http\Controllers\Controller;
use App\Libraries\CartChannel;
use App\Libraries\Coupon;
use App\Libraries\Shipping;
use App\Libraries\Status;
use App\Models\LogisticsMemory;
use App\Models\Product;
use App\Presenters\EcPay\LogisticPresenter;
use App\Repositories\OrderRepository;
use App\Repositories\SettingRepository;
use App\Repositories\UserAddressRepository;
use App\Services\Foreground\CartService;
use App\Services\Foreground\EcPay\LogisticService;
use App\Services\Foreground\LogisticsService;
use App\Services\Foreground\ProductService;
use App\Services\Foreground\OrderService;
use App\Services\Foreground\SettingService;
use App\ViewModels\Foreground\Cart\CheckIndexViewModel;
use App\ViewModels\Foreground\Cart\CheckoutIndexViewModel;
use App\ViewModels\Foreground\Cart\InfoIndexViewModel;
use Auth;
use Dflydev\DotAccessData\Data;
use Exception;
use Gloudemans\Shoppingcart\Facades\Cart;
use Illuminate\Contracts\View\Factory;
use Illuminate\Contracts\View\View;
use Illuminate\Foundation\Application;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Session\SessionManager;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use ValueError;

class CartController extends Controller
{
    public function infoIndex(Request $request, string $cartChannelValue = "normal")
    {
        $result = CartService::cartInfoPage($cartChannelValue);

        if (Status::isFails($result)) {
            abort(403, "請求無效");
        }

        $viewModel = new InfoIndexViewModel(
            $cartChannelValue
        );

        return view("foreground.cart.infoIndex", $viewModel);
    }

    public function errorMessageIndex(Request $request)
    {
        $message = $request->message;
        $cartChannelValue = $request->cartChannelValue;

        return redirect()->route("foreground.cart.infoIndex", [
            'cartChannelValue' => $cartChannelValue
        ])->with(Status::bladeFails($message));
    }

    public function checkIndex(Request $request, string $cartChannelValue)
    {
        if (Auth::guard("web")->check() === false) {
            return redirect()->route("foreground.login.index", [
                "redirect" => url()->full()
            ])->with(Status::bladeFails("請先登入會員"));
        }

        $result = CartService::checkPage(
            $cartChannelValue,
            $request->hash
        );


        if (Status::isFails($result)) {
            if ($result['code'] === "invalidShipping") {
                // dump("!!!請重新選擇運送方式!!!");
                return redirect()->route("foreground.cart.checkIndex", [
                    'cartChannelValue' => $cartChannelValue,
                ])->with(Status::bladeFails("請重新選擇運送方式"));
            } else {
                abort(403, "過程中發生錯誤，請重新操作: {$result['code']}");
            }
        }

        $settings = SettingRepository::getSettings();

        $viewModel = new CheckIndexViewModel(
            $result['data']['cart'],
            $request,
            $result['data']['cartChannel'],
            $result['data']['fromEcLogistics'],
            $result['data']['formData'],
            $result['data']['shippingAmount'],
            $result['data']['discountAmount'],
            $settings,
            $result['data']['userAddress'],
            $result['data']['markedCoupon'],
            $result['data']['member'],
        );

        return view("foreground.cart.checkIndex", $viewModel);
    }

    /**
     * 準備跳轉寄送方式
     */
    public function choiceLogisticsIndex(
        Request $request,
        string $cartChannelValue
    ) {
        Validator::make($request->all(), [
            "shippingMethods" => "required|in:selfPickUp,logistics",
            "name" => "required",
            "phone" => "required",
            "email" => "required|email",
            "payment" => "required",
            "comment" => "nullable",
        ])->validate();

        try {
            $channel = CartChannel::from($cartChannelValue);
        } catch (ValueError $e) {
            abort(403, "請求無效");
        }


        // 如果自取，那就夾帶 session 剛剛填寫的資料回到購物車頁面
        if ($request->selfPickUp === "1") {
            (new Shipping())->clear();

            $hash = CartService::setCarEncrypt(null, [
                'phone' => $request->phone,
                'name' => $request->name,
                'email' => $request->email,
                'payment' => $request->payment,
                'shippingMethods' => $request->shippingMethods,
            ]);

            return redirect()->route("foreground.cart.checkIndex", [
                'cartChannelValue' => $cartChannelValue,
                "hash" => $hash
            ]);
        }

        /**
         * 從綠界回到網站後要預選使用者已經填寫的資料，因此紀在 session 而不是網址
         * 因為網址若有 hash 太長會被綠界擋掉
         * 這裡務必在 config/session 設定 same_site 改為 null 才能正常運作
         */
        $this->setSessionForChoiceLogistics($request);
        // 這是提供給綠界物流指定回到網站以後，再回到下一個網址
        $redirect = route("foreground.logistics.resultIndex", [
            'cartChannelValue' => $request->cartChannelValue,
        ]);

        $mixAddress = "{$request->county}{$request->district}{$request->address}";

        return redirect()->route("foreground.logistics.redirectIndex", [
            'redirect' => $redirect,
            'userName' => $request->name,
            'userPhone' => $request->phone,
            'payment' => $request->payment,
            'shippingMethods' => $request->shippingMethods,
            'receiverZipCode' => $request->zipCode,
            'receiverAddress' => $mixAddress,
        ]);
    }

    /**
     * 建立訂單，才導向下一個頁面顯示
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     * @throws Exception
     */
    public function createOrderIndex(Request $request)
    {
        Validator::make($request->all(), [
            "cartChannelValue" => "required|in:normal,groupBuying,limitTimePreorder",
            "name" => "required",
            "phone" => "required",
            "email" => "required|email",
            "payment" => "required|in:Credit,ATM",
            "shippingType" => "required|in:selfPickUp,shipping",
            "couponCode" => "nullable",
            "couponQuantity" => "nullable",
            // "zipCode" => "nullable",
            // "county" => "nullable",
            // "district" => "nullable",
            // "address" => "nullable",
        ], [
            "cartChannelValue.required" => "請指定購買商品通道",
            "cartChannelValue.in" => "請指定購買商品通道",
            "name.required" => "請填寫姓名",
            "phone.required" => "請填寫電話",
            "email.required" => "請填寫 Email",
            "email.email" => "Email 格式錯誤",
            "payment.required" => "請選擇付款方式",
            "shippingType.required" => "請選擇寄送方式",
            "shippingType.in" => "請選擇寄送方式",
        ])->validate();


        $cartChannel = CartChannel::from($request->cartChannelValue);

        $userInfo = [
            'name' => $request->name,
            'phone' => $request->phone,
            'email' => $request->email,
        ];


        $decodeFromEcLogistics = CartService::getEcLogisticsFromSession();

        $isSelfPickUp = (string)$request->shippingType === "selfPickUp";


        $shippingTemperatureForSystem = ShippingTemperatureForSystem::from($request->checkedThisGroup);

        $result = OrderService::insert(
            $cartChannel,
            $userInfo,
            $request->payment,
            null,
            $decodeFromEcLogistics,
            $isSelfPickUp,
            $shippingTemperatureForSystem
        );


        if (Status::isFails($result)) {
            // 因為須要調整購物車的數量狀況，所以暫存使用者的基本資料，避免重新填寫
            if ($result['code'] === "notAllowConditions") {
                $messageFormat = "{$result['data']['messages'][0]}";
                session([
                    'rememberCartCheckUserInfo' => [
                        'name' => $request->name,
                        'phone' => $request->phone,
                        'email' => $request->email,
                    ],
                ]);

                // 如果是從優惠活動來的就要反回到優惠活動。否則就是回到購買商品確認的頁面 (步驟一)
                // dd("如果是從優惠活動來的就要反回到優惠活動", $messageFormat, $request->cartChannelValue);
                if ($request->cartChannelValue === "specialPromotion") {
                    // dd(Status::bladeFails($messageFormat));
                    return redirect()->route("foreground.specialPromotion.index")
                        ->with(Status::bladeFails($messageFormat));
                } else {
                    return redirect()->route("foreground.cart.infoIndex", [
                        "cartChannelValue" => $request->cartChannelValue,
                    ])->with(Status::bladeFails($messageFormat));
                }
            } elseif ($result['code'] === "fails") {
                return redirect()->route("foreground.cart.infoIndex", [
                    "cartChannelValue" => $request->cartChannelValue,
                ])->with(Status::bladeFails($result['message']));
            }

            Log::error("建立訂單失敗", [
                'request' => $request->all(),
                'result' => $result,
            ]);
            abort(500, "系統遇到錯誤，請與我們聯繫", $result['code']);
        }

        return redirect()->route("foreground.cart.checkoutIndex", [
            'orderNo' => $result['data']['order']->order_no,
        ]);
    }

    /**
     * @param  Request  $request
     * @param  string  $orderNo
     * @param  string|null  $type  如果須要列印頁面，請帶入字串 "print"
     * @return \Illuminate\Contracts\Foundation\Application|Factory|View|Application
     */
    public function checkoutIndex(Request $request, string $orderNo, ?string $type = null)
    {
        Validator::make([
            "orderNo" => $orderNo,
            "type" => $type,
        ], [
            "orderNo" => "required",
            "type" => "nullable|in:print",
        ], [
            "orderNo.required" => "請指定訂單編號",
            "type.in" => "請指定列印類型",
        ])->validate();


        $result = OrderService::getOrderByOrderNo($orderNo);

        if (Status::isFails($result)) {
            abort(404);
        }

        $order = $result['data']['order'];

        // 產生前往付款連結
        $paymentLink = OrderService::generatePaymentLink($order);

        // dump($result['data']['order']->toArray());

        $viewModel = new CheckoutIndexViewModel(
            $order,
            $paymentLink,
            $result['data']['calculate'],
        );


        // 如果要列印訂單
        if ($type === "print") {
            return view("foreground.cart.checkoutPrintIndex", $viewModel);
        }

        return view("foreground.cart.checkoutIndex", $viewModel);
    }

    /**
     * 將商品加入購物車
     */
    public function store(Request $request)
    {
        Validator::make($request->all(), [
            "type" => "required|in:buyNow,addToCart",
            "channel" => "required|in:normal,groupBuying,limitTimePreorder",
            "productId" => "required",
            "quantity" => "required|integer|min:1|max:99",
            "productVariantAttributeId" => "required",
            "redirect" => "required|url",
        ], [
            "type.required" => "請選擇加入購物車或直接購買",
            "type.in" => "請選擇加入購物車或直接購買",
            "channel.required" => "請指定商品通道",
            "productId.required" => "請選擇商品",
            "quantity.required" => "請選擇商品數量",
            "quantity.integer" => "商品數量必須為整數",
            "quantity.min" => "商品數量最少為1",
            "quantity.max" => "商品數量最多為99",
            "productVariantAttributeId.required" => "請選擇商品規格屬性",
            "redirect.required" => "必須要導向網址",
            "redirect.url" => "導向網址必須要網址",
        ])->validate();

        $result = CartService::add(
            $request->channel,
            $request->type,
            $request->productId,
            $request->quantity,
            $request->productVariantAttributeId,
            false
        );

        if (Status::isFails($result)) {
            return redirect()->away($request->redirect)
                ->with(Status::bladeFails($result['message']));
        }

        // 不同功能導向不同網址
        if ($request->type === "buyNow") {
            return redirect()->route("foreground.cart.infoIndex", [
                'cartChannelValue' => $request->channel,
                'redirect' => $request->redirect,
            ]);
        }

        // // 回到原來的地方。這個留著，須要可以啟用
        // return redirect()->away($request->redirect)
        //     ->with(Status::bladeSuccess("成功加入購物車"));

        return to_route("foreground.cart.infoIndex", [
            'cartChannelValue' => $request->channel,
            'redirect' => $request->redirect,
        ]);
    }

    /**
     * 刪除購物車商品
     *
     * @param  Request  $request
     *
     * @return RedirectResponse
     */
    public function destroy(Request $request)
    {
        Validator::make($request->all(), [
            "rowId" => "required",
            'channel' => 'required|in:normal,groupBuying,limitTimePreorder',
        ], [
            "rowId.required" => "請選擇要刪除的商品",
            "channel.required" => "請指定商品通道",
            "channel.in" => "請指定商品通道",
        ])->validate();

        CartService::delete($request->rowId, $request->channel);

        return redirect()->back()->with(Status::bladeSuccess("成功刪除商品"));
    }

    /**
     * 更新購物車的內容，目前是數量
     *
     * @param  Request  $request
     *
     * @return array
     */
    public function update(Request $request)
    {
        Validator::make($request->all(), [
            "rowId" => "required",
            "quantity" => "nullable|integer|min:-1|max:1",
            'channel' => 'required|in:normal,groupBuying,limitTimePreorder',
        ], [
            "rowId.required" => "請選擇要刪除的商品",
            "quantity.integer" => "商品數量必須為整數",
            "quantity.min" => "商品數量最少為-1",
            "quantity.max" => "商品數量最多為1",
            "channel.required" => "請指定商品通道",
            "channel.in" => "請指定商品通道",
        ])->validate();

        $isChecked = $this->getIsChecked($request);

        $result = CartService::update(
            $request->rowId,
            $request->channel,
            $request->quantity,
            $isChecked
        );

        return $result;
    }

    /**
     * 自取需要移除已設定的運費
     *
     * @param  Request  $request
     * @param  string  $cartChannelValue
     *
     * @return RedirectResponse
     */
    public function selfPickUpIndex(Request $request, string $cartChannelValue)
    {
        $shipping = new Shipping();
        $shipping->clear();

        return redirect()->route("foreground.cart.checkIndex", [
            'cartChannelValue' => $cartChannelValue,
        ]);
    }

    /**
     * 如果不帶 isChecked 參數，預設為 true
     *
     * @param  Request  $request
     *
     * @return bool
     */
    protected function getIsChecked(Request $request): bool
    {
        if (isset($request->isChecked)) {
            if ($request->isChecked === "true") {
                $isChecked = true;
            } else {
                $isChecked = false;
            }
        } else {
            $isChecked = true;
        }
        return $isChecked;
    }

    /**
     * 當使用者選擇物流方式後，將資料存入 session
     * @param  Request  $request
     * @return void
     */
    protected function setSessionForChoiceLogistics(Request $request): void
    {
        session([
            'formData' => [
                'name' => $request->name,
                'phone' => $request->phone,
                'email' => $request->email,
                'payment' => $request->payment,
                'shippingMethods' => $request->shippingMethods,

                // 加入地址
                'zipCode' => $request->zipCode,
                'county' => $request->county,
                'district' => $request->district,
                'address' => $request->address,
            ]
        ]);
    }

    public function choiceLogisticsWithWindowIndex(Request $request)
    {
        // 準備前往綠界物流
        $logisticService = new LogisticService();
        echo $logisticService->getRedirect(
            tempLogisticsID: 0,
            goodsAmount: $request->goodsAmount ?? 100,
            goodsName: config("website.name")."商品",
            senderName: SettingService::getSettingByKey('EcPayLogisticsSenderName')->value,

            // TODO 這裡暫時使用固定的值，之後要改成設定檔。因為六位數字的郵遞區號，在綠界會拋錯
            // senderZipCode: SettingService::getSettingByKey('EcPayLogisticsSenderZipCode')->value,
            senderZipCode: "11560",

            senderAddress: SettingService::getSettingByKey('EcPayLogisticsSenderAddress')->value,
            isCollection: "N",
            remark: $request->remark ?? null,
            temperature: $request->temperature ?? "0001",
            specification: $request->specification ?? "0001",
            scheduledPickupTime: $request->scheduledPickupTime ?? "4",
            receiverAddress: $request->receiverAddress ?? null,
            receiverZipCode: $request->receiverZipCode ?? null,
            receiverCellPhone: $request->userPhone ?? null,
            receiverPhone: $request->userTel ?? null,
            receiverName: $request->userName ?? null,
            enableSelectDeliveryTime: $request->enableSelectDeliveryTime ?? "N",
            eshopMemberID: $request->eshopMemberID ?? null,
            serverReplyURL: route("foreground.logistics.serverReplayIndex"),
            clientReplyURL: route("foreground.logistics.clientReplayIndex", ["redirect" => $request->redirect])
        );
    }

    /**
     * 綠界物流回傳到本網站 {@see route("foreground.logistics.clientReplayIndex")} 後，會將資料夾帶到 query string 並來到此處
     * @param  Request  $request
     * @return \Illuminate\Contracts\Foundation\Application|Factory|View|Application
     * @throws Exception
     */
    public function choiceLogisticsWithWindowCallbackIndex(Request $request)
    {
        // 驗證 query string 須要有 fromEcLogistics
        $validator = Validator::make($request->all(), [
            'fromEcLogistics' => 'required',
        ], [
            'fromEcLogistics.required' => 'fromEcLogistics 參數不存在',
        ]);

        if ($validator->errors()->count() > 0) {
            abort(403, $validator->errors()->first());
        }

        // 將資料存入 session
        $fromEcLogistics = decrypt($request->fromEcLogistics);

        // 記錄到運費類別，這裡並不判斷與商品免運的邏輯，因為這裡只是記錄運費的原始數據
        $title = LogisticPresenter::getSubType(
            $fromEcLogistics['LogisticsSubType']
        );

        $price = LogisticsService::getPriceFromSetting(
            $fromEcLogistics['LogisticsType']
        );

        (new Shipping())->add($title, $price, false);


        CartService::setEcLogisticsToSession($fromEcLogistics);

        // 從子視窗操作 parent 後，可以自動關閉子視窗
        return view("foreground.cart.choiceLogisticsWithWindowCallbackIndex");
    }

    public function orderConfirmedIndex()
    {
        return view("foreground.cart.orderConfirmedIndex");
    }
}
