<?php

namespace App\Http\Controllers\Foreground;

use App\Enums\PaymentType;
use App\Events\OrderPaid;
use App\Http\Controllers\Controller;
use App\Libraries\Status;
use App\Models\OrderTradeNumber;
use App\Repositories\OrderRepository;
use App\Repositories\OrderTradeNumberRepository;
use App\Services\Foreground\EcPay\PaymentService;
use Ecpay\Sdk\Factories\Factory;
use Ecpay\Sdk\Services\UrlService;
use Exception;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;

class PaymentController extends Controller
{
    public function exampleIndex()
    {
        $link = route('foreground.payment.redirectIndex');
        return "<a href='{$link}'>前往測試</a>";
    }

    public function redirectIndex(Request $request)
    {
        Validator::make($request->all(), [
            'hash' => 'required',
        ], [
            'hash.required' => 'hash 為必填',
        ])->validate();

        try {
            $decrypt = decrypt($request->hash);
        } catch (Exception $e) {
            abort(400, "hash 驗證失敗");
        }

        // 驗證付款名稱 for 金流
        $choosePayment = PaymentType::from($decrypt['paymentType'])->value;


        $paymentService = new PaymentService();
        echo $paymentService->getAutoSubmitForm(
            $decrypt['orderTradeNumber'],
            $decrypt['amount'],
            "訂單商品編號：{$decrypt['orderNo']}",
            $decrypt['itemName'],
            $choosePayment,
            route("foreground.payment.callbackIndex"),
            route("foreground.payment.orderResultIndex"),
            route("foreground.home.index"),
        );
    }

    /**
     * 從綠界後台可以模擬如信用卡的付款通知
     *
     * @param  Request  $request
     *
     * @return bool
     */
    public function callbackIndex(Request $request)
    {
        // 紀錄 log 方便追蹤
        Log::info("接收綠界付款結果資料", $request->all());

        $merchantTradeNo = $request->MerchantTradeNo;
        $orderTradeNumber
            = OrderTradeNumberRepository::getOrderByOrderTradeNumber($merchantTradeNo);

        if (empty($orderTradeNumber)) {
            Log::warning("找不到訂單", [$request->all()]);
            return false;
        }

        $order = $orderTradeNumber->order;

        if ( ! in_array($order->payment_status, ["unpaid", "authorized"])) {
            Log::warning("訂單狀態不是未付款或已授權",
                ["訂單狀態目前是：{$order->payment_status}"]);
            return false;
        }

        // 驗證綠界的
        $response
            = PaymentService::getResponseFromEcPay($request->MerchantTradeNo);
        Log::debug("向綠界求證金額", $response);

        // 檢查金額是否相符
        if ((int) $response['TradeAmt'] !== (int) $order->amount) {
            Log::warning("金額不符", [
                '綠界金額' => $response['TradeAmt'],
                '訂單金額' => $order->amount,
            ]);
            return false;
        }

        // 修改訂單狀態
        $order = OrderRepository::updatePaymentStatus(
            $order->order_no,
            'paid'
        );
        Log::info("訂單狀態修改為已付款", [$order->toArray()]);

        // 觸發事件
        event(new OrderPaid($order));

        echo "1|OK";
    }

    public function orderResultIndex(Request $request)
    {
        Log::info("orderResultIndex", [$request->all()]);

        $result = PaymentService::orderResultIndex(
            $request->all(),
        );

        // dd($result['data']['order']['order_no']);

        return redirect()->route("foreground.cart.checkoutIndex", [
            'orderNo' => $result['data']['order']['order_no']
        ]);
    }

    /**
     * 產生與綠界交易用的貿易編號
     *
     * @return RedirectResponse
     */
    public function generateTradeNoIndex(Request $request)
    {
        Validator::make($request->all(), [
            'hash' => 'required',
        ], [
            'hash.required' => '網址格式錯誤',
        ])->validate();

        try {
            $box = decrypt($request->hash);
        } catch (Exception $e) {
            abort(400, "hash 驗證失敗");
        }

        $model
            = OrderTradeNumberRepository::insert($box['orderNo'],
            $box['orderId']);

        $box['orderTradeNumber'] = $model->trade_no;

        $hash = encrypt($box);

        return redirect()->route('foreground.payment.redirectIndex', [
            'hash' => $hash
        ]);
    }
}
