<?php

namespace App\Http\Controllers\Foreground;

use App\Libraries\Status;
use App\Models\User;
use App\ViewModels\Foreground\ResetPasswordViewModel;
use Illuminate\Auth\Events\PasswordReset;
use Illuminate\Contracts\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Password;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use Illuminate\View\View;

class ResetPassword
{
    protected $rules = [
        'email' => ['required', 'email'],
        'password' => [
            'required',
            'confirmed',
            'min:6',
            'regex: /^(?=.*[a-zA-Z])(?=.*\d).+$/'
        ],
        'token' => ['required'],
    ];

    protected $message = [
        'password.regex' => '請輸入6位(含)以上英文和數字組合密碼',
        'password.required' => '請填寫密碼',
        'password.min' => '密碼最少須要 6 位數',
        'password.confirmed' => '密碼驗證錯誤',
        'email.required' => '請填寫電子郵件',
        'email.email' => '請填寫正確的電子郵件',
        'token.required' => 'token 錯誤，請重新操作',
    ];


    public function create()
    {
        return view("foreground.reset-password.create");
    }

    /**
     * 重設詢問表單
     * 參考官方 @link https://laravel.com/docs/10.x/passwords#password-reset-link-handling-the-form-submission
     *
     * @param  Request  $request
     * @return RedirectResponse|void
     */
    public function store(Request $request)
    {
        $request->validate(['email' => 'required|email']);

        $status = Password::sendResetLink($request->only('email'));

        if (Password::RESET_LINK_SENT === $status) {
            return back()->with(Status::bladeSuccess("如果您的資料比對無誤，我們將發送重設連結到 {$request->email}"));
        } elseif (Password::INVALID_USER === $status) {
            // 之所以這麼顯示，是避免安全猜測漏洞
            return back()->with(Status::bladeSuccess("如果您的資料比對無誤，我們將發送重設連結到 {$request->email}"));
        } elseif (Password::RESET_THROTTLED === $status) {
            return back()->with(Status::bladeFails("若未收到，請於一分鐘後重新點擊送出"));
        } else {
            $message = "系統錯誤，無法將密碼重設信件送出，請與我們聯繫";
            Log::error($message, compact('status'));
            return back()->with(Status::bladeFails($message));
        }
    }

    /**
     * 準備重新設定
     *
     * @param  Request  $request
     * @param  string  $token
     * @return Application|Factory|View
     */
    public function edit(Request $request, string $token)
    {
        if (empty($request->email)) {
            abort("403", "操作錯誤，您的連結不合法");
        }

        $resetPasswordViewModel = new ResetPasswordViewModel($token, $request->email);
        return view("foreground.reset-password.edit", $resetPasswordViewModel);
    }

    /**
     * 執行重新設定密碼
     * 注意照理須要與註冊的時候規則相同
     * @param  Request  $request
     * @return RedirectResponse
     * @see SignupController::store
     *
     * 這裡的實做方式可參考官方
     * @link https://laravel.com/docs/10.x/passwords#password-customization
     *
     */
    public function update(Request $request)
    {
        Validator::make([
            'email' => $request->email,
            'password' => $request->password,
            'password_confirmation' => $request->password_confirmation,
            'token' => $request->token,
        ], $this->rules, $this->message)
            // 依然使用預設的重新導向較方便
            ->validate();

        $credentials = $request->only('email', 'password', 'password_confirmation', 'token');

        // 如果使用者存在，會進入到 callback
        $status = Password::reset($credentials, function (User $user, string $password) {
            $user->forceFill([
                'password' => Hash::make($password)
            ])->setRememberToken(Str::random(60));

            $user->save();

            event(new PasswordReset($user));
        });

        if ($status === Password::INVALID_TOKEN) {
            Log::warning("使用者 {$request->email} 重設密碼時，token 錯誤，可能 token 已經被使用過了或是其他錯誤的可能",
                compact('status'));
            return redirect()->route("foreground.resetPassword.create")
                ->with(Status::bladeFails("Email 連結驗證已經失效，請重新操作密碼重設信件"));
        }


        return $status === Password::PASSWORD_RESET
            ? redirect()->route('foreground.login.index')->with(Status::bladeSuccess("密碼設定完成，請使用新密碼登入"))
            : back()->with(Status::bladeFails("密碼重設失敗，請再重新嘗試"));
    }
}
